@@ -41,7 +41,7 @@ my @ARGS = ({
     help => "-V, --version\n   Print version information",
   }, {
     spec => 'extra-opts:s@',
-    help => "--extra-opts=[section][\@file]\n   Read options from an ini file. See http://nagiosplugins.org/extra-opts\n   for usage and examples.",
+    help => "--extra-opts=[section][\@file]\n   Read options from an ini file. See https://www.monitoring-plugins.org/doc/extra-opts.html\n   for usage and examples.",
   }, {
     spec => 'timeout|t=i',
     help => "-t, --timeout=INTEGER\n   Seconds before plugin times out (default: %s)",
@@ -81,23 +81,34 @@ sub _spec_to_help
 {
   my ($self, $spec, $label) = @_;
 
-  my ($opts, $type) = split /=/, $spec, 2;
+  my ($opts, $type) = split /=|:|!/, $spec, 2;
+  my $optional = ($spec =~ m/:/);
+  my $boolean = ($spec =~ m/!/);
   my (@short, @long);
   for (split /\|/, $opts) {
     if (length $_ == 1) {
       push @short, "-$_";
     } else {
-      push @long, "--$_";
+      push @long, $boolean ? "--[no-]$_" : "--$_";
     }
   }
 
   my $help = join(', ', @short, @long);
   if ($type) {
-    if ($label) {
-      $help .= '=' . $label;
+    if (!$label) {
+      if ($type eq 'i' || $type eq '+' || $type =~ /\d+/) {
+        $label = 'INTEGER';
+      }
+      else {
+        $label = 'STRING';
+      }
+    }
+
+    if ($optional) {
+      $help .= '[=' . $label . ']';
     }
     else {
-      $help .= $type eq 'i' ? '=INTEGER' : '=STRING';
+      $help .= '=' . $label;
     }
   }
   elsif ($label) {
@@ -142,9 +153,10 @@ sub _options
     # Add help_string to @options
     if ($help_string =~ m/%s/) {
       my $default = defined $arg->{default} ? $arg->{default} : '';
-      # We only handle '%s' formats here, so escape everything else
-      $help_string =~ s/%(?!s)/%%/g;
-      push @options, sprintf($help_string, $default, $default, $default, $default);
+      # We only handle '%s' formats here
+      my $replaced = $help_string;
+      $replaced =~ s|%s|$default|gmx;
+      push @options, $replaced;
     } else {
       push @options, $help_string;
     }
@@ -154,10 +166,11 @@ sub _options
 }
 
 # Output for plugin -? (or missing/invalid args)
-sub _usage
-{
-  my $self = shift;
-  sprintf $self->_attr('usage'), $self->{_attr}->{plugin};
+sub _usage {
+  my $self  = shift;
+  my $usage = $self->_attr('usage');
+  $usage    =~ s|%s|$self->{_attr}->{plugin}|gmx;
+  return($usage);
 }
 
 # Output for plugin -V
@@ -195,7 +208,7 @@ sub _process_specs_getopt_long
     # Setup names and defaults
     my $spec = $arg->{spec};
     # Use first arg as name (like Getopt::Long does)
-    $spec =~ s/[=:].*$//;
+    $spec =~ s/[=:!].*$//;
     my $name = (split /\s*\|\s*/, $spec)[0];
     $arg->{name} = $name;
     if (defined $self->{$name}) {
@@ -247,7 +260,9 @@ sub _load_config_section
 
   my $Config;
   eval { $Config = Monitoring::Plugin::Config->read($file); };
-  $self->_die($@) if ($@); #TODO: add test?
+  $self->_die($@) if ($@);
+  defined $Config
+      or $self->_die(Monitoring::Plugin::Config->errstr);
 
   # TODO: is this check sane? Does --extra-opts=foo require a [foo] section?
   ## Nevertheless, if we die as UNKNOWN here we should do the same on default
@@ -369,27 +384,24 @@ sub arg
   my $self = shift;
   my %args;
 
-  # Named args
-  if ($_[0] =~ m/^(spec|help|required|default)$/ && scalar(@_) % 2 == 0) {
-    %args = validate( @_, {
-      spec => 1,
-      help => 1,
-      default => 0,
+  # Param name to required boolean
+  my %params = (
+      spec     => 1,
+      help     => 1,
+      default  => 0,
       required => 0,
-      label => 0,
-    });
+      label    => 0,
+  );
+
+  # Named args
+  if (exists $params{$_[0]} && @_ % 2 == 0) {
+    %args = validate( @_, \%params );
   }
 
   # Positional args
   else {
-    my @args = validate_pos(@_, 1, 1, 0, 0, 0);
-    %args = (
-      spec      => $args[0],
-      help      => $args[1],
-      default   => $args[2],
-      required  => $args[3],
-      label     => $args[4],
-    );
+    my @order = qw(spec help default required label);
+    @args{@order} = validate_pos(@_, @params{@order});
   }
 
   # Add to private args arrayref
@@ -433,7 +445,7 @@ sub getopts
   # Setup default alarm handler for alarm($ng->timeout) in plugin
   $SIG{ALRM} = sub {
     my $plugin = uc $self->{_attr}->{plugin};
-    $plugin =~ s/^check_//;
+    $plugin =~ s/^CHECK[-_]//i;
     $self->_die(
       sprintf("%s UNKNOWN - plugin timed out (timeout %ss)",
         $plugin, $self->timeout));
@@ -524,8 +536,8 @@ processing for Nagios plugins
   $ng->getopts;
 
   # Access arguments using named accessors or or via the generic get()
-  print $ng->warning;
-  print $ng->get('critical');
+  print $ng->opts->warning;
+  print $ng->opts->get('critical');
 
 
 
@@ -685,7 +697,8 @@ but basically it is a series of one or more argument names for this argument
 (separated by '|'), suffixed with an '=<type>' indicator if the argument
 takes a value. '=s' indicates a string argument; '=i' indicates an integer
 argument; appending an '@' indicates multiple such arguments are accepted;
-and so on. The following are some examples:
+appending an '!' indicates negation using '--no'-prefix is possible; and so on.
+The following are some examples:
 
 =over 4
 
@@ -697,6 +710,8 @@ and so on. The following are some examples:
 
 =item exclude|X=s@
 
+=item perfdata!
+
 =item verbose|v+
 
 =back
